;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Boom Boom, by dahnamics
;;
;; Credit: mikeyk (Shy Guy), Yoshicookiezeus (Thwomp boss v. 2.0), Schwa (sledge_ex)
;;
;; Description: I've listened to people's suggestions and I've decided 
;; to work on a Boom Boom sprite.  This sprite does not perfectly emulate
;; his behavior in SMB3 but it is a very close approximation.  I even used
;; the extra bit to determine whether he is the one just on the ground or the one
;; that flies into the air.  Check the readme to further understand how to insert the sprite.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Shy Guy, by mikeyk
;;
;; Description: A Shy Guy with many available configurations.
;;
;; Note: When rideable, clipping tables values should be: 03 0A FE 0E
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Uses first extra bit: NO
;;
;; Extra Property Byte 1
;;    bit 0 - enable spin killing
;;    bit 1 - stay on ledges
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		SPRITE_Y_SPEED  = $AA
		SPRITE_X_SPEED  = $B6
		SPRITE_Y_POS    = $D8
		H_OFFSCREEN     = $15A0
		V_OFFSCREEN     = $186C
		FREEZE_TIMER    = $1540
		SMASH_STATUS	= $1602
		STATE_TIMER 	= $163E

		EXTRA_BITS	= $7FAB10
		EXTRA_PROP_1	= $7FAB28
		Invuln_Timer	= $1564		; The RAM address for Clawgrip's stun duration
		MAX_Y_SPEED     = $3E

; definitions of bits (don't change these)
		IS_ON_GROUND    = $04
		TIME_ON_GROUND	= $A0
		RISE_SPEED	= $C0

		SPRITE_GRAVITY 	= $20
		SPRITE_GRAVITY2 = $04

		X_SPEED		dcb $08,$F8,$10,$F0
		X_SPEED2	dcb $10,$F0
		X_SPEED3	dcb $20,$E0
		KILLED_X_SPEED	dcb $F0,$10
		SPRITE_STATE    = $C2
		HIT_POINTS = $03        	; being stomped by Mario
		SPHERE = $6F

PROPFLASH:
	dcb $04,$06,$08,$0C,$04,$06,$08,$0C

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		dcb "INIT"
		PHY
		JSR SUB_HORZ_POS
		TYA
		STA $157C,x
		PLY
		LDA $1588,x             ; if on the ground, reset the turn counter
		ORA #$04
		STA $1588,x             ; if on the ground, reset the turn counter
		RTL                 


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite code JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		dcb "MAIN"                                    
		PHB                     ; \
		PHK                     ;  | main sprite function, just calls local subroutine
		PLB                     ;  |
		JSR SPRITE_CODE_START   ;  |
		PLB                     ;  |
		RTL                     ; /


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main code 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

RETURN		RTS
SPRITE_CODE_START
		JSR SUB_GFX       ; graphics routine

		LDA $14C8,x             ; return if sprite status != 8

		CMP #$08                ; \ if status != 8, return
		BNE RETURN              ; /

		LDA $9D                 ; \ if sprites locked, return
		BNE RETURN              ; /

		JSR SUB_OFF_SCREEN_X3   ; handle off screen situation

DONE_WITH_SPEED
		JSL $018032             ; interact with sprites

		LDA $1528,x             ; \ if sprite hit counter == 3
		CMP #HIT_POINTS         ;  |   
		BEQ NO_CONTACT        ; /

MARIO_INTERACT	JSL $01A7DC		; interact with mario 
		BCC NO_CONTACT          ; return if no contact
		LDA $154C,x             ; \ if sprite invincibility timer > 0 ...
		BNE NO_CONTACT          ; /    ... goto NO_CONTACT
		LDA #$08                ; \ sprite invincibility timer = $08
		STA $154C,x             ; /
		LDA $7D                 ; \  if Mario's y speed < 10 ...
		CMP #$10                ;  }   ... sprite will DEAD Mario
		BMI SPRITE_WINS         ; /    

		LDA SPRITE_STATE,x	; return if sprite is invulnerable
		CMP #$06
		BEQ SPRITE_WINS

MARIO_WINS		

		JSR SUB_STOMP_PTS       ; give Mario points
		JSL $01AA33             ; set Mario speed
		JSL $01AB99             ; display contact graphic
		LDA #$28                ; \ sound effect
		STA $1DFC               ; /
		LDA #$48		    ; \ Set stunned timer
		STA $1564,x             ; /
		INC $1528,x             ; increment sprite hit counter
		LDA $1528,x             ; \ if sprite hit counter == 3
		CMP #HIT_POINTS         ;  |   
		BNE SMUSH_SPRITE        ; /

;		LDA #$07		
;		STA SPRITE_STATE,x			

SMUSH_SPRITE	LDA #$0C                ; \ time to show semi-squashed sprite = $0C
		STA $1FE2,x             ; /

		LDA $1528,x             ; \ if sprite hit counter == 3
		CMP #HIT_POINTS         ;  |   
		BEQ SPRITE_DEAD		; /
			
		LDA #$06
		STA SPRITE_STATE,x
		BRA NEW_RETURN
SPRITE_DEAD	LDA #$07
		STA SPRITE_STATE,x
			
NEW_RETURN	RTS                     ; return

SPRITE_WINS         
		LDA $1497               ; \ if Mario is invincible...
		ORA $187A               ;  }  ... or Mario on yoshi...
		BNE NO_CONTACT          ; /   ... return
		JSR SUB_HORZ_POS        ; \  set new sprite direction
		TYA                     ;  }  
		STA $157C,x             ; /
		JSL $00F5B7             ; hurt Mario

NO_CONTACT	LDA SPRITE_STATE,x     
		CMP #$01
		BEQ WALKING0
		CMP #$02
		BEQ RISING0
		CMP #$03
		BEQ FLYING0
		CMP #$04
		BEQ DIVING0
		CMP #$05
		BEQ RUNNING0
		CMP #$06
		BEQ HURT0
		CMP #$07
		BEQ DEAD0

;-----------------------------------------------------------------------------------------
; state 0
;-----------------------------------------------------------------------------------------

WAITING            
		LDA V_OFFSCREEN,x       ;fall if offscreen vertically
		BNE SET_WALKING
    
		LDA H_OFFSCREEN,x       ;return if offscreen horizontally
		BNE RETURN0

		JSR SUB_HORZ_POS        ;determine if mario is close and act accordingly
		TYA        
		STA $157C,x

		LDA $1588,x             ; \ if sprite is in contact with an object...
		AND #$03                ;  |
		BEQ NO_OBJ_CONTACT0     ;  |
		LDA $157C,x             ;  | flip the direction status
		EOR #$01                ;  |
		STA $157C,x             ; /

NO_OBJ_CONTACT0      
		LDA $1588,x             ; run the subroutine if the sprite is in the air...
		ORA $151C,x             ; ...and not already turning
		BNE ON_GROUND0           ;
		JSR SUB_CHANGE_DIR      ;
		LDA #$01                ; set that we're already turning
		STA $151C,x             ;

ON_GROUND0
		LDA $1588,x             ; if on the ground, reset the turn counter
		AND #$04
		BEQ RETURN0
		STZ $151C,x
		STZ $AA,x
		BRA X_TIME0
                    
FALLING0
		LDA $1588,x             ; if on the ground, reset the turn counter
		AND #$04
		BEQ RETURN0
		LDA #$10                ; \  y speed = 10
		STA $AA,x               ; /
                                        
X_TIME0		STZ $B6,x

		BCS RETURN0
		LDA $0F    
		CLC        
		ADC #$40   
		CMP #$80                
		BCS THWOMP_4
THWOMP_4
		LDA $0F
		CLC
		ADC #$24    
		CMP #$50    
		BCS RETURN0

SET_WALKING
		INC SPRITE_STATE,x          ;chage state to falling

WALKING0	BRA WALKING

RETURN0		JSL $01802A		; update position based on speed values
		RTS                     

RISING0		BRA RISING
FLYING0		BRA FLYING1
DIVING0	BRA DIVING1
RUNNING0	BRA RUNNING1
HURT0		BRA HURT1
DEAD0		BRA DEAD1

;-----------------------------------------------------------------------------------------
; state 1
;-----------------------------------------------------------------------------------------

WALKING

		LDA EXTRA_BITS,x
		AND #$04
		BEQ CLEAR

		LDA $1528,x
		CMP #$01
		BNE CLEAR

		INC SPRITE_STATE,x	; set state to rising
		BRA CONTINUE
CLEAR
		LDA $1528,x
		CMP #$02
		BNE CONTINUE
		LDA #$05		
		STA SPRITE_STATE,x	
		
				
CONTINUE
		LDA $1588,x             ; \ if sprite is in contact with an object...
		AND #$03                ;  |
		BEQ NO_OBJ_CONTACT      ;  |
		LDA $157C,x             ;  | flip the direction status
		EOR #$01                ;  |
		STA $157C,x             ; /

NO_OBJ_CONTACT      
		LDA $1588,x             ; run the subroutine if the sprite is in the air...
		ORA $151C,x             ; ...and not already turning
		BNE ON_GROUND           ;
		JSR SUB_CHANGE_DIR      ;
		LDA #$01                ; set that we're already turning
		STA $151C,x             ;

ON_GROUND	LDA $1588,x             ; if on the ground, reset the turn counter
		AND #$04
		BEQ IN_AIR
		STZ $151C,x
		STZ $AA,x
		BRA X_TIME
                    
FALLING		LDA $1588,x             ; if on the ground, reset the turn counter
		AND #$04
		BEQ IN_AIR
		LDA #$10                ; \  y speed = 10
		STA $AA,x               ; /
                                        
X_TIME		LDA $1528,x             ; \ set x speed based on total HP
		ASL
		CLC
		ADC $157C,x		; and direction
		TAY

		LDA X_SPEED,y           ;  |
		STA $B6,x               ; /
                                        
IN_AIR		JSL $01802A             ; update position based on speed values

RETURN2		RTS	

RISING1		BRA RISING
FLYING1		BRA FLYING2
DIVING1	BRA DIVING2
RUNNING1	BRA RUNNING2
HURT1		BRA HURT2
DEAD1		BRA DEAD2

;-----------------------------------------------------------------------------------------
; state 2
;-----------------------------------------------------------------------------------------

RISING				

		LDA FREEZE_TIMER,x	;if we're still waiting on the ground, return
		BNE RETURN7

		LDA #$08
		STA $151C,x

		LDA SPRITE_Y_POS,x      ;check if the sprite is in original position
		CMP $151C,x
		BNE RISE

		LDA #$80		
		STA STATE_TIMER,x
			
		INC SPRITE_STATE,x	; set state to flying
		RTS			 

FLYING2		BRA FLYING

RISE		LDY $1528,x
		LDA #RISE_SPEED		;set rising speed and apply it
		STA SPRITE_Y_SPEED,x
		JSL $01801A             
RETURN7		RTS			 

DIVING2	BRA DIVING3
RUNNING2	BRA RUNNING3
HURT2		BRA HURT3
DEAD2		BRA DEAD3

;-----------------------------------------------------------------------------------------
; state 3
;-----------------------------------------------------------------------------------------

FLYING		LDA V_OFFSCREEN,x	;fall if offscreen vertically
		BNE SET_FALLING
			
		LDA H_OFFSCREEN,x	;return if offscreen horizontally
		BNE RETURN3
			
		STZ SPRITE_Y_SPEED,x

		LDA $1588,x             ; \ if sprite is in contact with an object...
		AND #$03                ;  |
		BEQ NO_OBJ_CONTACT2      ;  |
		LDA $157C,x             ;  | flip the direction status
		EOR #$01                ;  |
		STA $157C,x             ; /

NO_OBJ_CONTACT2
		LDY $157C,x             ; \ set x speed based on direction
		LDA X_SPEED2,y           ;  |
		STA $B6,x               ; /

		LDA STATE_TIMER,x
		CMP #$28                ; |   just go to normal walking code
		BCS RETURN3          ; /
                    
		LDA Invuln_Timer,x
		CMP #$00
		BNE RETURN3

		LDA STATE_TIMER,x
		CMP #$21
		BNE NO_JUMP2
		LDA $1570,x
		LSR A
		AND #01
		BEQ NO_JUMP2
		STZ STATE_TIMER,x					; x speed = 0                  

		BRA RETURN3
NO_JUMP2	CMP #$00
		BNE RETURN3

SET_FALLING	;STZ SPRITE_X_SPEED,x

		INC SPRITE_STATE,x	;chage state to falling

DIVING3	BRA DIVING

		LDA #$00     
		STA SPRITE_Y_SPEED,x	;set initial speed

RETURN3		JSL $01802A		;apply speed
		RTS	

RUNNING3	BRA RUNNING4
HURT3		BRA HURT
DEAD3		BRA DEAD4		 

;-----------------------------------------------------------------------------------------
; state 4
;-----------------------------------------------------------------------------------------

DIVING	JSL $01801A		;apply speed
			
		LDA SPRITE_Y_SPEED,x	;increase speed if below the max
		CMP #MAX_Y_SPEED
		BCS DONT_INC_SPEED2
		ADC #SPRITE_GRAVITY2
		STA SPRITE_Y_SPEED,x    
DONT_INC_SPEED2
		JSL $019138		;interact with objects
			
		LDA $1588,x	;return if not on the ground
		AND #IS_ON_GROUND
		BEQ RETURN69
		    
		JSR SUB_HORZ_POS        ; \ always face Mario
		TYA                     ;  |
		STA $157C,x             ; /

		LDA #TIME_ON_GROUND	;set time to stay on ground
		STA FREEZE_TIMER,x  
									
		LDA #$02		; go back to rising		
		STA SPRITE_STATE,x	

RUNNING4	BRA RUNNING

RETURN69	RTS     	 

HURT4		BRA HURT
DEAD4		BRA DEAD5		 

;-----------------------------------------------------------------------------------------
; state 5
;-----------------------------------------------------------------------------------------

RUNNING

		LDA $1588,x             ; \ if sprite is in contact with an object...
		AND #$03                ;  |
		BEQ NO_OBJ_CONTACT3      ;  |
		LDA $157C,x             ;  | flip the direction status
		EOR #$01                ;  |
		STA $157C,x             ; /

NO_OBJ_CONTACT3
		LDA $1588,x             ; run the subroutine if the sprite is in the air...
		ORA $151C,x             ; ...and not already turning
		BNE ON_GROUND3           ;
		JSR SUB_CHANGE_DIR      ;
		LDA #$01                ; set that we're already turning
		STA $151C,x             ;

ON_GROUND3	LDA $1588,x             ; if on the ground, reset the turn counter
		AND #$04
		BEQ NOTONGROUND
		STZ $151C,x
		STZ $AA,x
		BRA X_TIME3
                    
FALLING3	LDA $1588,x             ; if on the ground, reset the turn counter
		AND #$04
		BEQ NOTONGROUND
		LDA #$10                ; \  y speed = 10
		STA $AA,x               ; /
                                        
X_TIME3		LDY $157C,x             ; \ set x speed based on direction
		LDA X_SPEED3,y           ;  |
		STA $B6,x               ; /           ; /
                                        
NOTONGROUND              JSL $01802A             ; update position based on speed values

RETURN4		RTS	

DEAD5		BRA DEAD

;-----------------------------------------------------------------------------------------
; state 6
;-----------------------------------------------------------------------------------------

HURT		LDA #$00     
		STA SPRITE_Y_SPEED,x	;set initial speed

		JSL $01801A		;apply speed
			
		LDA SPRITE_Y_SPEED,x	;increase speed if below the max
		CMP #MAX_Y_SPEED
		BCS DONT_INC_SPEED
		ADC #SPRITE_GRAVITY2
		STA SPRITE_Y_SPEED,x    
DONT_INC_SPEED
		JSL $019138		;interact with objects
		
		LDA $1564,x	; return if sprite is invulnerable
		CMP #$00
		BNE RETURN68

		LDA #$01		
		STA SPRITE_STATE,x	

RETURN68	RTS     	 

;-----------------------------------------------------------------------------------------
; state 7
;-----------------------------------------------------------------------------------------

DEAD		LDA FREEZE_TIMER,x
		BNE RETURN10

		STZ SPRITE_X_SPEED,x

		LDA $0F30
		BNE RETURN10

		LDA $0F33
		AND $01
		BNE RETURN10

		INC SMASH_STATUS,X
		LDA SMASH_STATUS,x
		CMP #$02
		BNE RETURN10

		LDA #$10		;\ sound effect
		STA $1DF9		;/

		LDA #$50		; \	Sprite appears at set
		STA $D8,x		; /	y position

		LDA #SPHERE
		STA $7FAB9E,x
		LDA #$08
		STA $14C8,x
		JSL $07F7D2
		JSL $0187A7
		LDA #$08
		STA $7FAB10,x
	
RETURN10	RTS	

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PROPERTIES: dcb $7F,$3F

;The tables must now have 16 bytes.
;THE LAST 8 ARE ONLY CREATED BECAUSE OF XDISP.

;0-4 BYTE   - FRAME 1 RIGHT
;4-8 BYTE   - FRAME 2 RIGHT
;8-12 BYTE  - FRAME 1 LEFT
;12-16 BYTE - FRAME 2 LEFT

TILEMAP:
		dcb $08,$0A,$28,$2A ; ATTACK 1 	;\ RIGHT
		dcb $0C,$0E,$2C,$2E ; ATTACK 2 	;/ RIGHT
		dcb $40,$42,$60,$62 ; ATTACK 3 	;\ RIGHT
		dcb $44,$46,$64,$66 ; ATTACK 4 	;/ RIGHT
		dcb $00,$02,$20,$22 ; WAIT 1 	;\ RIGHT
		dcb $00,$02,$20,$22 ; WAIT 2 	;/ RIGHT
		dcb $00,$02,$20,$22 ; WAIT 3 	;\ RIGHT
		dcb $00,$02,$20,$22 ; WAIT 4 	;/ RIGHT
		dcb $00,$02,$20,$22 ; HURT 1 	;\ RIGHT
		dcb $00,$02,$20,$22 ; HURT 2 	;/ RIGHT
		dcb $00,$02,$20,$22 ; HURT 3 	;\ RIGHT
		dcb $00,$02,$20,$22 ; HURT 4 	;/ RIGHT
		dcb $80,$82,$A0,$A2 ; FLY 1 	;\ RIGHT
		dcb $8C,$8E,$AC,$AE ; FLY 2 	;/ RIGHT
		dcb $84,$86,$A4,$A6 ; FLY 3 	;\ RIGHT
		dcb $88,$8A,$A8,$AA ; FLY 4 	;/ RIGHT
		dcb $4C,$4E,$6C,$6E ; DEAD 1 	;\ RIGHT
		dcb $4C,$4E,$6C,$6E ; DEAD 2 	;/ RIGHT
		dcb $4C,$4E,$6C,$6E ; DEAD 3 	;\ RIGHT
		dcb $4C,$4E,$6C,$6E ; DEAD 4 	;/ RIGHT

		dcb $08,$0A,$28,$2A ; ATTACK 1 	;\ LEFT
		dcb $0C,$0E,$2C,$2E ; ATTACK 2 	;/ LEFT
		dcb $40,$42,$60,$62 ; ATTACK 3 	;\ LEFT
		dcb $44,$46,$64,$66 ; ATTACK 4 	;/ LEFT
		dcb $00,$02,$20,$22 ; WAIT 1 	;\ LEFT
		dcb $00,$02,$20,$22 ; WAIT 2 	;/ LEFT
		dcb $00,$02,$20,$22 ; WAIT 3 	;\ LEFT
		dcb $00,$02,$20,$22 ; WAIT 4 	;/ LEFT
		dcb $00,$02,$20,$22 ; HURT 1 	;\ LEFT
		dcb $00,$02,$20,$22 ; HURT 2 	;/ LEFT
		dcb $00,$02,$20,$22 ; HURT 3 	;\ LEFT
		dcb $00,$02,$20,$22 ; HURT 4 	;/ LEFT
		dcb $80,$82,$A0,$A2 ; FLY 1 	;\ LEFT
		dcb $8C,$8E,$AC,$AE ; FLY 2 	;/ LEFT
		dcb $84,$86,$A4,$A6 ; FLY 3 	;\ LEFT
		dcb $88,$8A,$A8,$AA ; FLY 4 	;/ LEFT
		dcb $4C,$4E,$6C,$6E ; DEAD 1 	;\ LEFT
		dcb $4C,$4E,$6C,$6E ; DEAD 2 	;/ LEFT
		dcb $4C,$4E,$6C,$6E ; DEAD 3 	;\ LEFT
		dcb $4C,$4E,$6C,$6E ; DEAD 4 	;/ LEFT

YDISP: 	
		dcb $F0,$F0,$00,$00 ; ATTACK 1 	;\ RIGHT
		dcb $F0,$F0,$00,$00 ; ATTACK 2 	;/ RIGHT
		dcb $F0,$F0,$00,$00 ; ATTACK 3 	;\ RIGHT
		dcb $F0,$F0,$00,$00 ; ATTACK 4 	;/ RIGHT
		dcb $F0,$F0,$00,$00 ; WAIT 1 	;\ RIGHT
		dcb $F0,$F0,$00,$00 ; WAIT 2 	;/ RIGHT
		dcb $F0,$F0,$00,$00 ; WAIT 3 	;\ RIGHT
		dcb $F0,$F0,$00,$00 ; WAIT 4 	;/ RIGHT
		dcb $F0,$F0,$00,$00 ; HURT 1 	;\ RIGHT
		dcb $F0,$F0,$00,$00 ; HURT 2 	;/ RIGHT
		dcb $F0,$F0,$00,$00 ; HURT 3 	;\ RIGHT
		dcb $F0,$F0,$00,$00 ; HURT 4 	;/ RIGHT
		dcb $F0,$F0,$00,$00 ; FLY 1 	;\ RIGHT
		dcb $F0,$F0,$00,$00 ; FLY 2 	;/ RIGHT
		dcb $F0,$F0,$00,$00 ; FLY 3 	;\ RIGHT
		dcb $F0,$F0,$00,$00 ; FLY 4 	;/ RIGHT
		dcb $F0,$F0,$00,$00 ; DEAD 1 	;\ RIGHT
		dcb $F0,$F0,$00,$00 ; DEAD 2 	;/ RIGHT
		dcb $F0,$F0,$00,$00 ; DEAD 3 	;\ RIGHT
		dcb $F0,$F0,$00,$00 ; DEAD 4 	;/ RIGHT

		dcb $F0,$F0,$00,$00 ; ATTACK 1 	;\ LEFT
		dcb $F0,$F0,$00,$00 ; ATTACK 2 	;/ LEFT
		dcb $F0,$F0,$00,$00 ; ATTACK 3 	;\ LEFT
		dcb $F0,$F0,$00,$00 ; ATTACK 4 	;/ LEFT
		dcb $F0,$F0,$00,$00 ; WAIT 1 	;\ LEFT
		dcb $F0,$F0,$00,$00 ; WAIT 2 	;/ LEFT
		dcb $F0,$F0,$00,$00 ; WAIT 3 	;\ LEFT
		dcb $F0,$F0,$00,$00 ; WAIT 4 	;/ LEFT
		dcb $F0,$F0,$00,$00 ; HURT 1 	;\ LEFT
		dcb $F0,$F0,$00,$00 ; HURT 2 	;/ LEFT
		dcb $F0,$F0,$00,$00 ; HURT 3 	;\ LEFT
		dcb $F0,$F0,$00,$00 ; HURT 4 	;/ LEFT
		dcb $F0,$F0,$00,$00 ; FLY 1 	;\ LEFT
		dcb $F0,$F0,$00,$00 ; FLY 2 	;/ LEFT
		dcb $F0,$F0,$00,$00 ; FLY 3 	;\ LEFT
		dcb $F0,$F0,$00,$00 ; FLY 4 	;/ LEFT
		dcb $F0,$F0,$00,$00 ; DEAD 1 	;\ LEFT
		dcb $F0,$F0,$00,$00 ; DEAD 2 	;/ LEFT
		dcb $F0,$F0,$00,$00 ; DEAD 3 	;\ LEFT
		dcb $F0,$F0,$00,$00 ; DEAD 4 	;/ LEFT

XDISP: 	
		dcb $F8,$08,$F8,$08 ; ATTACK 1 	;\ RIGHT
		dcb $F8,$08,$F8,$08 ; ATTACK 2 	;/ RIGHT
		dcb $F8,$08,$F8,$08 ; ATTACK 3 	;\ RIGHT
		dcb $F8,$08,$F8,$08 ; ATTACK 4 	;/ RIGHT
		dcb $F8,$08,$F8,$08 ; WAIT 1 	;\ RIGHT
		dcb $F8,$08,$F8,$08 ; WAIT 2 	;/ RIGHT
		dcb $F8,$08,$F8,$08 ; WAIT 3 	;\ RIGHT
		dcb $F8,$08,$F8,$08 ; WAIT 4 	;/ RIGHT
		dcb $F8,$08,$F8,$08 ; HURT 1 	;\ RIGHT
		dcb $F8,$08,$F8,$08 ; HURT 2 	;/ RIGHT
		dcb $F8,$08,$F8,$08 ; HURT 3 	;\ RIGHT
		dcb $F8,$08,$F8,$08 ; HURT 4 	;/ RIGHT
		dcb $F8,$08,$F8,$08 ; FLY 1 	;\ RIGHT
		dcb $F8,$08,$F8,$08 ; FLY 2 	;/ RIGHT
		dcb $F8,$08,$F8,$08 ; FLY 3 	;\ RIGHT
		dcb $F8,$08,$F8,$08 ; FLY 4 	;/ RIGHT
		dcb $F8,$08,$F8,$08 ; DEAD 1 	;\ RIGHT
		dcb $F8,$08,$F8,$08 ; DEAD 2 	;/ RIGHT
		dcb $F8,$08,$F8,$08 ; DEAD 3 	;\ RIGHT
		dcb $F8,$08,$F8,$08 ; DEAD 4 	;/ RIGHT

		dcb $08,$F8,$08,$F8 ; ATTACK 1 	;\ LEFT
		dcb $08,$F8,$08,$F8 ; ATTACK 2 	;/ LEFT
		dcb $08,$F8,$08,$F8 ; ATTACK 3 	;\ LEFT
		dcb $08,$F8,$08,$F8 ; ATTACK 4 	;/ LEFT
		dcb $08,$F8,$08,$F8 ; WAIT 1 	;\ LEFT
		dcb $08,$F8,$08,$F8 ; WAIT 2 	;/ LEFT
		dcb $08,$F8,$08,$F8 ; WAIT 3 	;\ LEFT
		dcb $08,$F8,$08,$F8 ; WAIT 4 	;/ LEFT
		dcb $08,$F8,$08,$F8 ; HURT 1 	;\ LEFT
		dcb $08,$F8,$08,$F8 ; HURT 2 	;/ LEFT
		dcb $08,$F8,$08,$F8 ; HURT 3 	;\ LEFT
		dcb $08,$F8,$08,$F8 ; HURT 4 	;/ LEFT
		dcb $08,$F8,$08,$F8 ; FLY 1 	;\ LEFT
		dcb $08,$F8,$08,$F8 ; FLY 2 	;/ LEFT
		dcb $08,$F8,$08,$F8 ; FLY 3 	;\ LEFT
		dcb $08,$F8,$08,$F8 ; FLY 4 	;/ LEFT
		dcb $08,$F8,$08,$F8 ; DEAD 1 	;\ LEFT
		dcb $08,$F8,$08,$F8 ; DEAD 2 	;/ LEFT
		dcb $08,$F8,$08,$F8 ; DEAD 3 	;\ LEFT
		dcb $08,$F8,$08,$F8 ; DEAD 4 	;/ LEFT

SUB_GFX:
		JSR GET_DRAW_INFO

		LDA $14			;\ Frame counter ..
		LSR			; | Add in frame animation rate; More LSRs for slower animation.
		LSR			; | Add in frame animation rate; More LSRs for slower animation.
		AND #$03		; | 01 means we animate between 4 frames (00 and 01).
		ASL A 			; | 
		ASL A			; | ASL x2 (0-4) makes it switch between the first byte and fifth byte,
		STA $03 		;/ i.e. first animation and second animation. The result is stored into $03.

IF_DEAD:
		LDA SPRITE_STATE,x		; If dead
		CMP #$07
		BNE IF_WAIT
		LDA $03
		CLC
		ADC #$40                ;    ...set killed frame
		STA $03			        
		BRA DONE_WALKING

IF_WAIT:
		LDA SPRITE_STATE,x		; If waiting
		CMP #$00
		BNE IF_RISE
		LDA $03					; |
		CLC						; |
		ADC #$10				; | ...use stun frames.
		STA $03					; /
		BRA DONE_WALKING

IF_RISE:
		LDA SPRITE_STATE,x		; If rising
		CMP #$02
		BNE IF_FLYING
		LDA $03					; |
		CLC						; |
		ADC #$30				; | ...use stun frames.
		STA $03					; /
		BRA DONE_WALKING

IF_FLYING:
		LDA SPRITE_STATE,x		; If flying
		CMP #$03
		BNE IF_DIVING
		LDA $03					; |
		CLC						; |
		ADC #$30				; | ...use stun frames.
		STA $03					; /
		BRA DONE_WALKING

IF_DIVING:
		LDA SPRITE_STATE,x		; If diving
		CMP #$04
		BNE IF_INVULN
		LDA $03					; |
		CLC						; |
		ADC #$30				; | ...use stun frames.
		STA $03					; /
		BRA DONE_WALKING

IF_INVULN:
		LDA SPRITE_STATE,x		; If invulnerable
		CMP #$06
		BNE DONE_WALKING
		LDA $03					; |
		CLC						; |
		ADC #$10				; | ...use stun frames.
		STA $03					; /
		BRA DONE_WALKING

DONE_WALKING:
		LDA $157C,x
		STA $02			; Store direction to $02 for use with property routine later.
		BNE NoAdd
		LDA $03			;\
		CLC			; | If sprite faces left ..
		ADC #$50		; | Adding 16 more bytes to the table.
		STA $03			;/ So we can invert XDISP to not mess up the sprite's appearance.
NoAdd:
		PHX			;\ Push sprite index ..
		LDX #$03		;/ And load X with number of tiles to loop through.
Loop:
		PHX			; Push number of tiles to loop through.
		TXA			;\
		ORA $03			;/ Transfer it to X and add in the "left displacement" if necessary.
		TAX			;\ Get it back into X for an index.

		LDA $00			;\
		CLC			; | Apply X displacement of the sprite.
		ADC XDISP,x		; |
		STA $0300,y		;/ 

		LDA $01			;\
		CLC			; | Y displacement is added for the Y position, so one tile is higher than the other.
		ADC YDISP,x		; | Otherwise, both tiles would have been drawn to the same position!
		STA $0301,y		; | If X is 00, i.e. first tile, then load the first value from the table and apply that
					;/ as the displacement. For the second tile, F0 is added to make it higher than the first.

		LDA TILEMAP,x
		STA $0302,y

		PHX			; Push number of times to go through loop + "left" displacement if necessary.
		LDX $02			;\
		LDA PROPERTIES,x	; | Set properties based on direction.
		LDX $15E9               ;  |
		ORA $15F6,x             ;  | get palette info
		ORA $64                 ;  | put in level t_PROPERTIES
		STA $0303,y             ; / store tile t_PROPERTIES

		LDA SPRITE_STATE,x		; If waiting
		CMP #$07
		BNE NO_FLASH

		LDA $14		; \
		AND #$07	  ;  | cycle through all 8 sprite palettes
		TAX		  ;  |
		LDA $0303,y	  ;  |
		AND #$F1	  ;  | <-- F1 = 11110001 = clear all Palette-related bits
		ORA PROPFLASH,x  ;  |
		STA $0303,y	  ; /

NO_FLASH:
		PLX			; Pull number of times to go through loop.

		INY			;\
		INY			; | The OAM is 8x8, but our sprite is 16x16 ..
		INY			; | So increment it 4 times.
		INY			;/
		
		PLX			; Pull current tile back.
		DEX			; After drawing this tile, decrease number of tiles to go through loop. If the second tile
					; is drawn, then loop again to draw the first tile.

		BPL Loop		; Loop until X becomes negative (FF).
	
		PLX			; Pull back the sprite index! We pushed it at the beginning of the routine.

		LDY #$02		; Y ends with the tile size .. 02 means it's 16x16
		LDA #$03		; A -> number of tiles drawn - 1.
					; I drew 2 tiles, so 2-1 = 1. A = 01.

		JSL $01B7B3		; Call the routine that draws the sprite.
		RTS			; Never forget this!       
                    

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_CHANGE_DIR
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $019098

SUB_CHANGE_DIR      ;LDA $15AC,x
                    ;BNE LABEL41
                    ;LDA #$08   
                    ;STA $15AC,x
                    LDA $B6,x  
                    EOR #$FF   
                    INC A      
                    STA $B6,x  
                    LDA $157C,x
                    EOR #$01   
                    STA $157C,x
LABEL41             RTS        



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; routines below can be shared by all sprites.  they are ripped from original
; SMW and poorly documented
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; points routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

STAR_SOUNDS         dcb $00,$13,$14,$15,$16,$17,$18,$19

SUB_STOMP_PTS       PHY                     ; 
                    LDA $1697               ; \
                    CLC                     ;  | 
                    ADC $1626,x             ; / some enemies give higher pts/1ups quicker??
                    INC $1697               ; increase consecutive enemies stomped
                    TAY                     ;
                    INY                     ;
                    CPY #$08                ; \ if consecutive enemies stomped >= 8 ...
                    BCS NO_SOUND            ; /    ... don't play sound 
                    LDA STAR_SOUNDS,y       ; \ play sound effect
                    STA $1DF9               ; /   
NO_SOUND            TYA                     ; \
                    CMP #$08                ;  | if consecutive enemies stomped >= 8, reset to 8
                    BCC NO_RESET            ;  |
                    LDA #$08                ; /
NO_RESET            JSL $02ACE5             ; give mario points
                    PLY                     ;
                    RTS                     ; return
                
                    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B75C

TABLE1              dcb $0C,$1C
TABLE2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC TABLE1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA TABLE2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B829 - vertical mario/sprite position check - shared
; Y = 1 if mario below sprite??
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B829 

SUB_VERT_POS        LDY #$00               ;A:25A1 X:0007 Y:0001 D:0000 DB:03 S:01EA P:envMXdizCHC:0130 VC:085 00 FL:924
                    LDA $96                ;A:25A1 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdiZCHC:0146 VC:085 00 FL:924
                    SEC                    ;A:2546 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizCHC:0170 VC:085 00 FL:924
                    SBC $D8,x              ;A:2546 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizCHC:0184 VC:085 00 FL:924
                    STA $0F                ;A:25D6 X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0214 VC:085 00 FL:924
                    LDA $97                ;A:25D6 X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0238 VC:085 00 FL:924
                    SBC $14D4,x            ;A:2501 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizcHC:0262 VC:085 00 FL:924
                    BPL LABEL11            ;A:25FF X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0294 VC:085 00 FL:924
                    INY                    ;A:25FF X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0310 VC:085 00 FL:924
LABEL11             RTS                    ;A:25FF X:0007 Y:0001 D:0000 DB:03 S:01EA P:envMXdizcHC:0324 VC:085 00 FL:924


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B817 - horizontal mario/sprite check - shared
; Y = 1 if mario left of sprite??
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B817        

SUB_HORZ_POS        LDY #$00                ;A:25D0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1020 VC:097 00 FL:31642
                    LDA $94                 ;A:25D0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZCHC:1036 VC:097 00 FL:31642
                    SEC                     ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1060 VC:097 00 FL:31642
                    SBC $E4,x               ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1074 VC:097 00 FL:31642
                    STA $0F                 ;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1104 VC:097 00 FL:31642
                    LDA $95                 ;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1128 VC:097 00 FL:31642
                    SBC $14E0,x             ;A:2500 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZcHC:1152 VC:097 00 FL:31642
                    BPL LABEL16             ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1184 VC:097 00 FL:31642
                    INY                     ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1200 VC:097 00 FL:31642
LABEL16             RTS                     ;A:25FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:1214 VC:097 00 FL:31642


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B85D - off screen processing code - shared
; sprites enter at different points
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B83B             

TABLE3              dcb $40,$B0
TABLE6              dcb $01,$FF 
TABLE4              dcb $30,$C0,$A0,$80,$A0,$40,$60,$B0 
TABLE5              dcb $01,$FF,$01,$FF,$01,$00,$01,$FF

SUB_OFF_SCREEN_X0   LDA #$06                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | 
SUB_OFF_SCREEN_X1   LDA #$04                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X2   LDA #$02                ;  |
STORE_03            STA $03                 ;  |
                    BRA START_SUB           ;  |
SUB_OFF_SCREEN_X3   STZ $03                 ; /

START_SUB           JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_2            ; /    
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ;  |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA $D8,x               ; \
                    CLC                     ;  | 
                    ADC #$50                ;  | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ;  | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ;  | 
                    CMP #$02                ;  | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ;  |
                    BNE RETURN_2            ; /
                    LDA $13                 ; \ 
                    AND #$01                ;  | 
                    ORA $03                 ;  | 
                    STA $01                 ;  |
                    TAY                     ; /
                    LDA $1A                 ;x boundry ;A:0101 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0256 VC:090 00 FL:16953
                    CLC                     ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZcHC:0280 VC:090 00 FL:16953
                    ADC TABLE4,y            ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZcHC:0294 VC:090 00 FL:16953
                    ROL $00                 ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0326 VC:090 00 FL:16953
                    CMP $E4,x               ;x pos ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0364 VC:090 00 FL:16953
                    PHP                     ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0394 VC:090 00 FL:16953
                    LDA $1B                 ;x boundry hi ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01EC P:eNvMXdizCHC:0416 VC:090 00 FL:16953
                    LSR $00                 ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01EC P:envMXdiZCHC:0440 VC:090 00 FL:16953
                    ADC TABLE5,y            ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01EC P:envMXdizcHC:0478 VC:090 00 FL:16953
                    PLP                     ;A:01FF X:0006 Y:0001 D:0000 DB:03 S:01EC P:eNvMXdizcHC:0510 VC:090 00 FL:16953
                    SBC $14E0,x             ;x pos high ;A:01FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0538 VC:090 00 FL:16953
                    STA $00                 ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0570 VC:090 00 FL:16953
                    LSR $01                 ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0594 VC:090 00 FL:16953
                    BCC LABEL20             ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZCHC:0632 VC:090 00 FL:16953
                    EOR #$80                ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZCHC:0648 VC:090 00 FL:16953
                    STA $00                 ;A:017E X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0664 VC:090 00 FL:16953
LABEL20             LDA $00                 ;A:017E X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0688 VC:090 00 FL:16953
                    BPL RETURN_2            ;A:017E X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0712 VC:090 00 FL:16953
ERASE_SPRITE        LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ;  |
                    BCC KILL_SPRITE         ; /
                    LDY $161A,x             ;A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZCHC:0140 VC:071 00 FL:21152
                    CPY #$FF                ;A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0172 VC:071 00 FL:21152
                    BEQ KILL_SPRITE         ;A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0188 VC:071 00 FL:21152
                    LDA #$00                ; \ mark sprite to come back    A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0204 VC:071 00 FL:21152
                    STA $1938,y             ; /                             A:FF00 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZcHC:0220 VC:071 00 FL:21152
KILL_SPRITE         STZ $14C8,x             ; erase sprite
RETURN_2            RTS                     ; return

VERTICAL_LEVEL      LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ;  |
                    BNE RETURN_2            ; /
                    LDA $13                 ; \ only handle every other frame??
                    LSR A                   ;  | 
                    BCS RETURN_2            ; /
                    AND #$01                ;A:0227 X:0006 Y:00EC D:0000 DB:03 S:01ED P:envMXdizcHC:0228 VC:112 00 FL:1142
                    STA $01                 ;A:0201 X:0006 Y:00EC D:0000 DB:03 S:01ED P:envMXdizcHC:0244 VC:112 00 FL:1142
                    TAY                     ;A:0201 X:0006 Y:00EC D:0000 DB:03 S:01ED P:envMXdizcHC:0268 VC:112 00 FL:1142
                    LDA $1C                 ;A:0201 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0282 VC:112 00 FL:1142
                    CLC                     ;A:02BD X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0306 VC:112 00 FL:1142
                    ADC TABLE3,y            ;A:02BD X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0320 VC:112 00 FL:1142
                    ROL $00                 ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01ED P:enVMXdizCHC:0352 VC:112 00 FL:1142
                    CMP $D8,x               ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01ED P:enVMXdizCHC:0390 VC:112 00 FL:1142
                    PHP                     ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNVMXdizcHC:0420 VC:112 00 FL:1142
                    LDA.W $001D             ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01EC P:eNVMXdizcHC:0442 VC:112 00 FL:1142
                    LSR $00                 ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01EC P:enVMXdiZcHC:0474 VC:112 00 FL:1142
                    ADC TABLE6,y            ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01EC P:enVMXdizCHC:0512 VC:112 00 FL:1142
                    PLP                     ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01EC P:envMXdiZCHC:0544 VC:112 00 FL:1142
                    SBC $14D4,x             ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNVMXdizcHC:0572 VC:112 00 FL:1142
                    STA $00                 ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0604 VC:112 00 FL:1142
                    LDY $01                 ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0628 VC:112 00 FL:1142
                    BEQ LABEL22             ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0652 VC:112 00 FL:1142
                    EOR #$80                ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0668 VC:112 00 FL:1142
                    STA $00                 ;A:027F X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0684 VC:112 00 FL:1142
LABEL22             LDA $00                 ;A:027F X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0708 VC:112 00 FL:1142
                    BPL RETURN_2            ;A:027F X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0732 VC:112 00 FL:1142
                    BMI ERASE_SPRITE        ;A:0280 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0170 VC:064 00 FL:1195

SUB_IS_OFF_SCREEN   LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ;  |  
                    RTS                     ; / return
